setlocal textwidth=0
setlocal nospell
" setlocal spelllang=en
setlocal nofoldenable
setlocal formatoptions+=p
set formatprg=prettier\ --parser=markdown

function! MarkdownFold(lnum)
  let level = strlen(matchstr(getline(a:lnum), '^#'))
  if level > 0
    return '>'.level
  else
    return '='
  endif
endfunction

augroup fmt
  au BufWritePost *.md silent! :!prettier --parser=markdown %
augroup END

setlocal foldmethod=expr foldexpr=MarkdownFold(v:lnum)
setlocal foldtext=getline(v:foldstart)[1:30].'...'.repeat('\ ',999)

let b:match_words = b:match_words . ',^```.\+$:^```$'

iabbrev <buffer> =w []()<Esc><Left><Left><Left><Left>
iabbrev <buffer> =' ``<Esc><Left>
iabbrev <buffer> =" ```<CR><CR>```<Esc>2k4l

function! SelectCodeBlock(inner) abort
  call search('^```.\+', 'bcW')
  execute 'normal ' . (a:inner ==# 'i' ? 'jv' : 'v')
  call search('^```$', 'cW')
  execute 'normal ' . (a:inner ==# 'i' ? 'k$' : '$')
endfunction

xnoremap <silent> ac :<C-u>call SelectCodeBlock("a")<CR>
onoremap <silent> ac :<C-u>normal vac<CR>
xnoremap <silent> ic :<C-u>call SelectCodeBlock("i")<CR>
onoremap <silent> ic :<C-u>normal vic<CR>

nmap <buffer> gs :10 split term://pandoc -s --pdf-engine=lualatex --filter pandoc-fignos --filter pandoc-citeproc --listings % -o %:r.pdf && zathura %:r.pdf<cr>:startinsert<cr>
nmap <buffer> <silent> g= :!prettier --parser=markdown %<cr>:redraw!<cr>

nnoremap <silent> gr :call fzf#run({
                        \ 'source': 'bibtex-ls',
                        \ 'sink*': function('bibtex#bibtex_cite_sink'),
                        \ 'down': '10%',
                        \ 'options': '--ansi --multi --border top --prompt "Cite> "'})<CR>

inoremap <silent> <C-h> <c-g>u<c-o>:call fzf#run({
                        \ 'source': bibtex#bibtex_ls_in(),
                        \ 'sink*': function('bibtex#bibtex_cite_sink_insert'),
                        \ 'down': '10%',
                        \ 'options': '--ansi --multi --border top --prompt "Cite> "'})<CR>
