function PrintDiagnostics(opts, bufnr, line_nr, client_id)
  opts = opts or {}
  bufnr = bufnr or 0
  line_nr = line_nr or (vim.api.nvim_win_get_cursor(0)[1] - 1)
  local line_diagnostics = vim.lsp.diagnostic.get_line_diagnostics(bufnr, line_nr, opts, client_id)
  if vim.tbl_isempty(line_diagnostics) then
    return
  end
  local diagnostic_message = ""
  for _, diagnostic in ipairs(line_diagnostics) do
    diagnostic_message = diagnostic_message .. string.format("   ■ %s ", diagnostic.message or "")
  end
  local diagnostic_message_fmt = diagnostic_message:gsub("\n", " ")
  vim.api.nvim_buf_set_virtual_text(bufnr, 1, line_nr, { { diagnostic_message_fmt, "DiagnosticSignHint" } }, {})
end

function Clear(_, bufnr, _, _)
  bufnr = bufnr or 0
  vim.api.nvim_buf_clear_namespace(bufnr, 1, 0, -1)
end

vim.diagnostic.config({
  virtual_text = false,
  signs = false,
  underline = true,
  update_in_insert = false,
  severity_sort = false,
})

vim.cmd([[autocmd CursorHold * lua PrintDiagnostics()]])
vim.cmd([[autocmd CursorMoved * lua Clear()]])
