local default_opts = {
  noremap = true,
  silent = true,
  expr = false,
  nowait = false,
  script = false,
  unique = false,
}

local keymap = function(mode, keys, cmd, opts)
  local o = vim.tbl_deep_extend("force", default_opts, opts or {})
  vim.api.nvim_set_keymap(mode, keys, cmd, o)
end

-- leader keys
vim.g.mapleader = ","
vim.g.maplocalleader = ","

-- navigation
keymap("c", [[<C-a>]], [[<Home>]])
keymap("c", [[<C-e>]], [[<End>]])

-- buffers
keymap("n", [[-]], [[<cmd>Ex<cr>]])
keymap("n", [[<leader>!]], [[<cmd>10sp +te<cr>]])
keymap("n", [[<leader>.]], [[<cmd>lcd %:p:h<cr>]])
keymap("n", [[<Backspace>]], [[<cmd>bp <bar> bd! #<cr>]])
vim.cmd([[nnoremap <leader>e :e <C-R>=expand("%:p:h") . "/" <cr>]])

-- tabs
vim.cmd([[nnoremap <expr> <S-tab> winnr('$') ==? 2 ? "<C-w>T" : ":tabprev<cr>"]])
keymap("n", [[<leader><tab>]], [[<cmd>$tabnew<cr>]])
vim.cmd([[nnoremap <expr> <tab> tabpagenr('$') ==? 1 ? "<C-^>" : ":tabnext<cr>"]])

-- misc
keymap("c", [[W!!]], [[w !sudo tee % >/dev/null<cr>:e!<cr><cr>]]) -- write file as root
keymap("i", [[<C-s>]], [[<c-g>u<Esc>[s1z=`]a<c-g>u]]) -- fix spelling (last error, repeatable)
keymap("n", [[<F9>]], [[<cmd>make<cr>]]) -- barebone make
keymap("n", [[<leader>S]], [[vip:sort iu<cr>]]) -- sort in reverse lexicographic order
keymap("n", [[<leader>de]], [[:setlocal spell!<bar>setlocal spell?<cr>:setlocal spelllang=en<cr>]])
keymap("n", [[<leader>df]], [[:setlocal spell!<bar>setlocal spell?<cr>:setlocal spelllang=fr<cr>]])
keymap("n", [[<leader>p]], [[<cmd>set invpaste<cr>:set paste?<cr>]])
keymap("n", [[<leader>s]], [[vip:sort u<cr>]]) -- sort in lexicographic order (Credit: strager)
keymap("n", [[J]], [[mzJ`z]]) -- join lines, but keep cursor stationary
keymap("n", [[N]], [[Nzz]]) -- keep cursor centered on screen when looking behind
keymap("n", [[U]], [[<C-r>]]) -- redo
keymap("n", [[n]], [[nzz]]) -- keep cursor centered on screen when looking ahead
keymap("n", [[zs]], [[1z=]]) -- spell fix using first entry
keymap("v", [[<leader>S]], [[:sort iu<cr>]]) -- sort in reverse lexicographic order
keymap("v", [[<leader>s]], [[:sort u<cr>]]) -- sort in lexicographic order (Credit: strager)

-- poor man send-to-repl features (we need to fire a REPL in a split first)
vim.cmd([[noremap ss Vy<C-w>wpa<CR><C-\><C-n><C-w>pj]])
vim.cmd([[noremap s y<C-w>wpa<CR><C-\><C-n><C-w>p]])
vim.cmd([[xnoremap s y<C-w>wpa<CR><C-\><C-n><C-w>p]])

keymap("n", [[<C-p>]], [[<cmd>Files<cr>]])
keymap("n", [[<C-q>]], [[<cmd>call misc#quickfix_toggle()<cr>]])
keymap("n", [[<C-s>]], [[<cmd>RgCw<cr>]])
keymap("n", [[<C-t>]], [[<cmd>TagsCw<cr>]])
keymap("n", [[<leader>-]], [[:split term://TERM=xterm-mono lf<cr>:startinsert<cr>]])
keymap("n", [[<leader><leader>]], [[<cmd>Buffers<cr>]])
keymap("n", [[<leader>G]], [[:tabe term://TERM=xterm-mono tig<cr>:startinsert<cr>]])
keymap("n", [[<leader>gt]], [[:tabe term://TERM=xterm-mono tig %<cr>:startinsert<cr>]])
keymap("n", [[<leader>gb]], [[<cmd>BCommits<cr>]])
keymap("n", [[<leader>gc]], [[<cmd>Commits<cr>]])
keymap("n", [[<leader>gg]], [[<cmd>GFiles?<cr>]])
keymap("n", [[<leader>r]], [[<cmd>History<cr>]])
