-- Source mfussenegger/dotfiles, with little changes
-- and some ideas from https://nuxsh.is-a.dev/blog/custom-nvim-statusline.html
local M = {}
local api = vim.api

function M.file_or_lsp_status()
  local messages = vim.lsp.util.get_progress_messages()
  local mode = api.nvim_get_mode().mode

  if mode ~= "n" or vim.tbl_isempty(messages) then
    return M.format_uri(vim.uri_from_bufnr(api.nvim_get_current_buf()))
  end

  local percentage
  local result = {}

  for _, msg in pairs(messages) do
    if msg.message then
      table.insert(result, msg.title .. ": " .. msg.message)
    else
      table.insert(result, msg.title)
    end
    if msg.percentage then
      percentage = math.max(percentage or 0, msg.percentage)
    end
  end
  if percentage then
    return string.format("%03d: %s", percentage, table.concat(result, ", "))
  else
    return table.concat(result, ", ")
  end
end

function M.format_uri(uri)
  if vim.startswith(uri, "jdt://") then
    local package = uri:match("contents/[%a%d._-]+/([%a%d._-]+)") or ""
    local class = uri:match("contents/[%a%d._-]+/[%a%d._-]+/([%a%d$]+).class") or ""
    return string.format("%s::%s", package, class)
  else
    return vim.fn.fnamemodify(vim.uri_to_fname(uri), ":.")
  end
end

function M.diagnostic_status()
  local count = {}
  local levels = {
    errors = "Error",
    warnings = "Warn",
    infos = "Info",
    hints = "Hint",
  }

  for k, level in pairs(levels) do
    count[k] = vim.tbl_count(vim.diagnostic.get(0, { severity = level }))
  end

  local errors = ""
  local warnings = ""
  local infos = ""

  if count["errors"] ~= 0 then
    errors = " ● " .. count["errors"]
  end
  if count["warnings"] ~= 0 then
    warnings = " ◉ " .. count["warnings"]
  end
  if count["hints"] ~= 0 or count["infos"] ~= 0 then
    infos = " ○ " .. count["hints"] + count["infos"]
  end

  return errors .. warnings .. infos .. " "
end

function M.statusline()
  local parts = {
    [[ %{luaeval("require'utils.statusline'.file_or_lsp_status()")} %m%r ]],
    -- "%{gitbranch#name()}",
    -- '%{get(b:, "git_status", "") ? "" : "*"} ',
    [[%{luaeval("require'utils.statusline'.diagnostic_status()")} %=]],
    "%#warningmsg#",
    "%{&ff!='unix'?'['.&ff.']':''}",
    "%*",
    "%#warningmsg#",
    "%#DiagnosticSignWarn#%{(&fenc!='utf-8'&&&fenc!='')?'['.&fenc.']':''}",
    "%*",
    "%{&paste?'¶':''} ",
    "%{&spell?&spelllang:''} ",
    -- "[%{&tw}%{matchstr(&fo, 'w')}%{&cc?','.&cc:''}%{&et?',+':''}] ",
    "%l %P ",
  }
  return table.concat(parts, "")
end

return M
